#
# Copyright (c) 2006 Mellanox Technologies. All rights reserved.
#
# This Software is licensed under one of the following licenses:
#
# 1) under the terms of the "Common Public License 1.0" a copy of which is
#    available from the Open Source Initiative, see
#    http://www.opensource.org/licenses/cpl.php.
#
# 2) under the terms of the "The BSD License" a copy of which is
#    available from the Open Source Initiative, see
#    http://www.opensource.org/licenses/bsd-license.php.
#
# 3) under the terms of the "GNU General Public License (GPL) Version 2" a
#    copy of which is available from the Open Source Initiative, see
#    http://www.opensource.org/licenses/gpl-license.php.
#
# Licensee has the right to choose one of the above licenses.
#
# Redistributions of source code must retain the above copyright
# notice and one of the license notices.
#
# Redistributions in binary form must reproduce both the above copyright
# notice, one of the license notices in the documentation
# and/or other materials provided with the distribution.
#
#

%define KVERSION 2.6.32-400.29.2.el5uek
%define _name mlnx_en
%define _version 1.5.7

Name:		%{_name}-%{KVERSION}
Version:	%{_version}
Release:	2

Summary:	Mellonax Ethernet NIC Driver
Group:		System Environment/Base
License: 	GPL/BSD
Url:		http://Mellanox.com/
Source:		%{_name}-%{_version}.tgz
BuildRoot:	%{?build_root:%{build_root}}%{!?build_root:/var/tmp/MLNX_EN}
Vendor:		Oracle America, Inc

%define	MEMTRACK		0
%define	sysctl_update		1

%define builddef	1
%define builddebug	0
%define buildxen	0

%ifarch i686
%define karch		i686
%define builddebug	1
%define buildxen	0
%endif

%ifarch x86_64
%define karch		x86_64
%define builddebug	1
%define buildxen	0
%endif

%define MODS_DIR /lib/modules/%{KVERSION}/updates/kernel/drivers/net

# Disable debugging
%define debug_package %{nil}
%define __check_files %{nil}

# Disable brp-lib64-linux
%ifarch x86_64 ia64
%define __arch_install_post %{nil}
%endif

%if %{builddef}
BuildRequires:	kernel-uek-devel = %{KVERSION}
Requires:	perl
Requires:	kernel-uek = %{KVERSION}
%endif

%description
ConnectX Ehternet device driver

%if %{builddebug}
%package -n %{name}debug
Summary:	Mellonax Ethernet NIC Driver for the debug kernel
Group:		System Environment/Base
BuildRequires:	kernel-uek-debug-devel = %{KVERSION}
Requires:	perl
Requires:	kernel-uek-debug = %{KVERSION}

%description -n %{name}debug
ConnectX Ehternet device driver for the debug kernel

%endif 

%if %{buildxen}
%package -n %{name}xen
Summary:	Mellonax Ethernet NIC Driver for the Oracle VM kernel
Group:		System Environment/Base
BuildRequires:	kernel-ovs-devel = %{KVERSION}
Requires:	kernel-ovs = %{KVERSION}

%description -n %{name}xen
ConnectX Ehternet device driver for the Oracle VM kernel

%endif 

#################################################################################################################################
%prep
%setup -n %{_name}-%{_version}

###
### build
###
%build
rm -rf $RPM_BUILD_ROOT
cd $RPM_BUILD_DIR/%{_name}-%{_version}

MLNX_EN_PATCH_PARAMS=""

%if %{MEMTRACK}
MLNX_EN_PATCH_PARAMS=(${MLNX_EN_PATCH_PARAMS} " --with-memtrack")
%endif

BuildDriver()
{
  KV=$1
  Flavour=$2
  if [ x$Flavour = "xxen" ]; then
    Flavour=${Flavour:+${Flavour}}
  else
    Flavour=${Flavour:+.${Flavour}}
  fi

  if [ -d /usr/src/kernels/${KV}.%{karch} ]; then
    KSRC=/usr/src/kernels/${KV}.%{karch}${Flavour}
  elif [ -d /usr/src/kernels/${KV}${Flavour} ]; then
    KSRC=/usr/src/kernels/${KV}${Flavour}
  else
    echo "Unable to find kernel-uek-devel files"
    exit 1
  fi
    
  KVERSION=${KV}${Flavour} scripts/mlnx_en_patch.sh $MLNX_EN_PATCH_PARAMS
  cat config.mk | grep -v "^KSRC">config.mk~
  mv config.mk~ config.mk
  echo "KSRC:=${KSRC}" >> config.mk
  make
  gzip -c scripts/mlx4_en.7 > mlx4_en.7.gz

  LIB_MOD_DIR=/lib/modules/${KV}${Flavour}/updates
  LIB_MOD_DIR_NET=${LIB_MOD_DIR}/kernel/drivers/net

  mkdir -p $RPM_BUILD_ROOT/${LIB_MOD_DIR_NET}
  make install DESTDIR=$RPM_BUILD_ROOT KERNELRELEASE=${KV}${Flavour}
  install -D -m 644 mlx4_en.7.gz $RPM_BUILD_ROOT/usr/share/man/man7/mlx4_en.7.gz
  make clean_kernel
}

%if %{builddef}
BuildDriver %{KVERSION}
(
    cd $RPM_BUILD_ROOT
    find lib/modules/%{KVERSION}/updates -type f | sed 's:^:/:'
) > files-def
%endif

%if %{builddebug}
BuildDriver %{KVERSION} debug
(
  cd $RPM_BUILD_ROOT
  find lib/modules/%{KVERSION}.debug/updates -type f | sed 's:^:/:'
) > files-debug
%endif

%if %{buildxen}
BuildDriver %{KVERSION} xen
(
  cd $RPM_BUILD_ROOT
  find lib/modules/%{KVERSION}xen/updates -type f | sed 's:^:/:'
) > files-xen
%endif

mv $RPM_BUILD_ROOT $RPM_BUILD_ROOT.SAVE

%install
rm -rf $RPM_BUILD_ROOT
mv $RPM_BUILD_ROOT.SAVE $RPM_BUILD_ROOT
# Do nothing here

%clean
#Remove installed driver after rpm build finished
rm -rf $RPM_BUILD_ROOT

%pre -n %{name}

%post -n %{name}

#############################################################################################################
#                                       Modules configuration                                               #
#############################################################################################################
%if %{sysctl_update}

sysctl -q -w net.ipv4.tcp_timestamps=0
sysctl -q -w net.ipv4.tcp_sack=0
sysctl -q -w net.ipv4.tcp_low_latency=1
sysctl -q -w net.core.netdev_max_backlog=250000
sysctl -q -w net.core.rmem_max=16777216
sysctl -q -w net.core.wmem_max=16777216
sysctl -q -w net.core.rmem_default=16777216
sysctl -q -w net.core.wmem_default=16777216
sysctl -q -w net.core.optmem_max=16777216
sysctl -q -w net.ipv4.tcp_mem="16777216 16777216 16777216"
sysctl -q -w net.ipv4.tcp_rmem="4096 87380 16777216"
sysctl -q -w net.ipv4.tcp_wmem="4096 65536 16777216"

if [ -f /etc/sysctl.conf ]; then
perl -ni -e 'if (/\#\# MLXNET tuning parameters \#\#$/) { $filter = 1 }' -e 'if (!$filter) { print }' -e 'if (/\#\# END MLXNET \#\#$/){ $filter = 0 }' /etc/sysctl.conf

cat << EOF >> /etc/sysctl.conf
## MLXNET tuning parameters ##
net.ipv4.tcp_timestamps = 0
net.ipv4.tcp_sack = 0
net.ipv4.tcp_low_latency = 1
net.core.netdev_max_backlog = 250000
net.core.rmem_max = 16777216
net.core.wmem_max = 16777216
net.core.rmem_default = 16777216
net.core.wmem_default = 16777216
net.core.optmem_max = 16777216
net.ipv4.tcp_mem = 16777216 16777216 16777216
net.ipv4.tcp_rmem = 4096 87380 16777216
net.ipv4.tcp_wmem = 4096 65536 16777216
## END MLXNET ##
EOF
fi

%endif

/sbin/depmod -r -ae %{KVERSION}
if [ -f /usr/share/man/man7/mlx4_en.7.gz ]; then
        exit 0
fi

%if %{buildxen}
%post -n %{name}xen

#############################################################################################################
#                                       Modules configuration                                               #
#############################################################################################################
%if %{sysctl_update}

sysctl -q -w net.ipv4.tcp_timestamps=0
sysctl -q -w net.ipv4.tcp_sack=0
sysctl -q -w net.ipv4.tcp_low_latency=1
sysctl -q -w net.core.netdev_max_backlog=250000
sysctl -q -w net.core.rmem_max=16777216
sysctl -q -w net.core.wmem_max=16777216
sysctl -q -w net.core.rmem_default=16777216
sysctl -q -w net.core.wmem_default=16777216
sysctl -q -w net.core.optmem_max=16777216
sysctl -q -w net.ipv4.tcp_mem="16777216 16777216 16777216"
sysctl -q -w net.ipv4.tcp_rmem="4096 87380 16777216"
sysctl -q -w net.ipv4.tcp_wmem="4096 65536 16777216"

if [ -f /etc/sysctl.conf ]; then
    # Remove existed MLXNET tuning paramters then add new.
    # Original used perl to do this bug OVM did not included perl
    if grep -q "^## MLXNET tuning parameters ##$" /etc/sysctl.conf; then
      if  grep -q "^## END MLXNET ##$" /etc/sysctl.conf; then
           filter=0
           tmp_sysctl=/tmp/sysctl.conf.$$
           while read line; do
               if [ $filter -eq 1 ]; then
                 if echo $line | grep -q "^## END MLXNET ##$"; then
                   filter=0
                 fi
                 continue
               fi
               if echo $line | grep -q "^## MLXNET tuning parameters ##$" ; then
                 filter=1
                 continue
               fi
               echo $line >> $tmp_sysctl
           done < /etc/sysctl.conf
           mv $tmp_sysctl /etc/sysctl.conf
      fi
    fi

cat << EOF >> /etc/sysctl.conf
## MLXNET tuning parameters ##
net.ipv4.tcp_timestamps = 0
net.ipv4.tcp_sack = 0
net.ipv4.tcp_low_latency = 1
net.core.netdev_max_backlog = 250000
net.core.rmem_max = 16777216
net.core.wmem_max = 16777216
net.core.rmem_default = 16777216
net.core.wmem_default = 16777216
net.core.optmem_max = 16777216
net.ipv4.tcp_mem = 16777216 16777216 16777216
net.ipv4.tcp_rmem = 4096 87380 16777216
net.ipv4.tcp_wmem = 4096 65536 16777216
## END MLXNET ##
EOF
fi

%endif

/sbin/depmod -r -ae %{KVERSION}xen
if [ -f /usr/share/man/man7/mlx4_en.7.gz ]; then
        exit 0
fi
%endif

%if %{builddebug}
%post -n %{name}debug

#############################################################################################################
#                                       Modules configuration                                               #
#############################################################################################################
%if %{sysctl_update}

sysctl -q -w net.ipv4.tcp_timestamps=0
sysctl -q -w net.ipv4.tcp_sack=0
sysctl -q -w net.ipv4.tcp_low_latency=1
sysctl -q -w net.core.netdev_max_backlog=250000
sysctl -q -w net.core.rmem_max=16777216
sysctl -q -w net.core.wmem_max=16777216
sysctl -q -w net.core.rmem_default=16777216
sysctl -q -w net.core.wmem_default=16777216
sysctl -q -w net.core.optmem_max=16777216
sysctl -q -w net.ipv4.tcp_mem="16777216 16777216 16777216"
sysctl -q -w net.ipv4.tcp_rmem="4096 87380 16777216"
sysctl -q -w net.ipv4.tcp_wmem="4096 65536 16777216"

if [ -f /etc/sysctl.conf ]; then
perl -ni -e 'if (/\#\# MLXNET tuning parameters \#\#$/) { $filter = 1 }' -e 'if (!$filter) { print }' -e 'if (/\#\# END MLXNET \#\#$/){ $filter = 0 }' /etc/sysctl.conf

cat << EOF >> /etc/sysctl.conf
## MLXNET tuning parameters ##
net.ipv4.tcp_timestamps = 0
net.ipv4.tcp_sack = 0
net.ipv4.tcp_low_latency = 1
net.core.netdev_max_backlog = 250000
net.core.rmem_max = 16777216
net.core.wmem_max = 16777216
net.core.rmem_default = 16777216
net.core.wmem_default = 16777216
net.core.optmem_max = 16777216
net.ipv4.tcp_mem = 16777216 16777216 16777216
net.ipv4.tcp_rmem = 4096 87380 16777216
net.ipv4.tcp_wmem = 4096 65536 16777216
## END MLXNET ##
EOF
fi

%endif

/sbin/depmod -r -ae %{KVERSION}.debug
if [ -f /usr/share/man/man7/mlx4_en.7.gz ]; then
        exit 0
fi

%endif
#############################################################################################################

# END of post -n mlnx-en 

###
### preun section
###

%preun -n %{name}

###
### post uninstall section
###
%postun -n %{name}
if [ $1 = 0 ]; then  # 1 : Erase, not upgrade
        # Remove previous configuration if exist
        /sbin/depmod -r -ae %{KVERSION}


# Clean sysctl.conf
if [ -f /etc/sysctl.conf ]; then
perl -ni -e 'if (/\#\# MLXNET tuning parameters \#\#$/) { $filter = 1 }' -e 'if (!$filter) { print }' -e 'if (/\#\# END MLXNET \#\#$/){ $filter = 0 }' /etc/sysctl.conf
fi

fi

%if %{buildxen}
%postun -n %{name}xen
if [ $1 = 0 ]; then  # 1 : Erase, not upgrade
        # Remove previous configuration if exist
        /sbin/depmod -r -ae %{KVERSION}xen


# Clean sysctl.conf
if [ -f /etc/sysctl.conf ]; then
    # Remove existed MLXNET tuning paramters then add new.
    # Original used perl to do this bug OVM did not included perl
    if grep -q "^## MLXNET tuning parameters ##$" /etc/sysctl.conf; then
      if  grep -q "^## END MLXNET ##$" /etc/sysctl.conf; then
         filter=0
         tmp_sysctl=/tmp/sysctl.conf.$$
         while read line; do
             if [ $filter -eq 1 ]; then
               if echo $line | grep -q "^## END MLXNET ##$"; then
                 filter=0
               fi
               continue
             fi
             if echo $line | grep -q "^## MLXNET tuning parameters ##$" ; then
               filter=1
               continue
             fi
             echo $line >> $tmp_sysctl
         done < /etc/sysctl.conf
         mv $tmp_sysctl /etc/sysctl.conf
      fi
    fi
fi

fi
%endif

%if %{builddebug}
%postun -n %{name}debug
if [ $1 = 0 ]; then  # 1 : Erase, not upgrade
        # Remove previous configuration if exist
        /sbin/depmod -r -ae %{KVERSION}.debug


# Clean sysctl.conf
if [ -f /etc/sysctl.conf ]; then
perl -ni -e 'if (/\#\# MLXNET tuning parameters \#\#$/) { $filter = 1 }' -e 'if (!$filter) { print }' -e 'if (/\#\# END MLXNET \#\#$/){ $filter = 0 }' /etc/sysctl.conf
fi

fi
%endif
###
### Files
###
%if %{builddef}
%files -f files-def -n %{name}
%defattr(-,root,root)
#%{MODS_DIR}/mlx4
/usr/share/man/man7/mlx4_en.7.gz
%endif

%if %{builddebug}
%files -f files-debug -n %{name}debug
%defattr(-,root,root)
%endif

%if %{buildxen}
%files -f files-xen -n %{name}xen
%defattr(-,root,root)
/usr/share/man/man7/mlx4_en.7.gz
%endif

# END Files

%changelog
* Tue Jan 5 2012 Joe Jin <joe.jin@oracle.com>
- Fix incorrect kernel version string for depmod.
- Fix postinstall/postuninstall script failed.
* Wed Sep 8 2008 Vladimir Sokolovsky <vlad@mellanox.co.il>
- Added nfsrdma support
* Wed Aug 13 2008 Vladimir Sokolovsky <vlad@mellanox.co.il>
- Added mlx4_en support
* Tue Aug 21 2007 Vladimir Sokolovsky <vlad@mellanox.co.il>
- Added %build macro
* Sun Jan 28 2007 Vladimir Sokolovsky <vlad@mellanox.co.il>
- Created spec file for kernel-ib
